@@ -16,6 +16,7 @@
 from ctypes import Structure, c_char_p, c_uint, c_int, c_size_t, \
 		c_ssize_t, c_char, c_void_p, c_bool, create_string_buffer, \
 		POINTER as _POINTER, CDLL as _cdll, memmove as _memmove, byref as _byref
+from ctypes.util import find_library
 from os import strerror as _strerror
 from platform import system as _system
 import weakref
@@ -38,6 +39,15 @@ def _checkNegative(result, func, arguments):
 	else:
 		raise OSError(-result, _strerror(-result))
 
+# Python 2 and Python 3 compatible _isstring function.
+def _isstring(s):
+	try:
+		# attempt to evaluate basestring (Python 2)
+		return isinstance(s, basestring)
+	except NameError:
+		# No basestring --> Python 3
+		return isinstance(s, str)
+
 class _ScanContext(Structure):
 	pass
 class _ContextInfo(Structure):
@@ -58,8 +68,13 @@ _DevicePtr = _POINTER(_Device)
 _ChannelPtr = _POINTER(_Channel)
 _BufferPtr = _POINTER(_Buffer)
 
-_lib = _cdll('libiio.dll' if 'Windows' in _system() else 'libiio.so.0',
-		use_errno = True, use_last_error = True)
+if 'Windows' in _system():
+	_iiolib = 'libiio.dll'
+else:
+	# Non-windows, possibly Posix system
+	_iiolib = 'iio'
+
+_lib = _cdll(find_library(_iiolib), use_errno = True, use_last_error = True)
 
 _get_backends_count = _lib.iio_get_backends_count
 _get_backends_count.restype = c_uint
@@ -732,7 +747,7 @@ class Context(object):
 
 		if(_context is None):
 			self._context = _new_default()
-		elif type(_context) is str or type(_context) is unicode:
+		elif _isstring(_context):
 			self._context = _new_uri(_context.encode('ascii'))
 		else:
 			self._context = _context
