@@ -16,6 +16,7 @@
  *
  * */
 
+#include "iio-config.h"
 #include "iio-private.h"
 
 #include <errno.h>
@@ -44,7 +45,8 @@ struct iio_buffer * iio_device_create_buffer(const struct iio_device *dev,
 	int ret = -EINVAL;
 	struct iio_buffer *buf;
 	unsigned int sample_size = iio_device_get_sample_size(dev);
-	if (!sample_size)
+
+	if (!sample_size || !samples_count)
 		goto err_set_errno;
 
 	buf = malloc(sizeof(*buf));
@@ -199,11 +201,12 @@ ssize_t iio_buffer_foreach_sample(struct iio_buffer *buffer,
 			void *, size_t, void *), void *d)
 {
 	uintptr_t ptr = (uintptr_t) buffer->buffer,
+		  start = ptr,
 		  end = ptr + buffer->data_length;
 	const struct iio_device *dev = buffer->dev;
 	ssize_t processed = 0;
 
-	if (buffer->sample_size <= 0)
+	if (buffer->sample_size == 0)
 		return -EINVAL;
 
 	if (buffer->data_length < buffer->dev_sample_size)
@@ -220,14 +223,14 @@ ssize_t iio_buffer_foreach_sample(struct iio_buffer *buffer,
 				break;
 
 			/* Test if the buffer has samples for this channel */
-			if (!TEST_BIT(buffer->mask, chn->index))
+			if (!TEST_BIT(buffer->mask, chn->number))
 				continue;
 
-			if (ptr % length)
-				ptr += length - (ptr % length);
+			if ((ptr - start) % length)
+				ptr += length - ((ptr - start) % length);
 
 			/* Test if the client wants samples from this channel */
-			if (TEST_BIT(dev->mask, chn->index)) {
+			if (TEST_BIT(dev->mask, chn->number)) {
 				ssize_t ret = callback(chn,
 						(void *) ptr, length, d);
 				if (ret < 0)
@@ -236,7 +239,9 @@ ssize_t iio_buffer_foreach_sample(struct iio_buffer *buffer,
 					processed += ret;
 			}
 
-			ptr += length;
+			if (i == dev->nb_channels - 1 || dev->channels[
+					i + 1]->index != chn->index)
+				ptr += length * chn->format.repeat;
 		}
 	}
 	return processed;
@@ -252,31 +257,36 @@ void * iio_buffer_first(const struct iio_buffer *buffer,
 {
 	size_t len;
 	unsigned int i;
-	uintptr_t ptr = (uintptr_t) buffer->buffer;
+	uintptr_t ptr = (uintptr_t) buffer->buffer,
+		  start = ptr;
 
 	if (!iio_channel_is_enabled(chn))
 		return iio_buffer_end(buffer);
 
 	for (i = 0; i < buffer->dev->nb_channels; i++) {
 		struct iio_channel *cur = buffer->dev->channels[i];
-		len = cur->format.length / 8;
+		len = cur->format.length / 8 * cur->format.repeat;
 
 		/* NOTE: dev->channels are ordered by index */
 		if (cur->index < 0 || cur->index == chn->index)
 			break;
 
 		/* Test if the buffer has samples for this channel */
-		if (!TEST_BIT(buffer->mask, cur->index))
+		if (!TEST_BIT(buffer->mask, cur->number))
 			continue;
 
-		if (ptr % len)
-			ptr += len - (ptr % len);
+		/* Two channels with the same index use the same samples */
+		if (i > 0 && cur->index == buffer->dev->channels[i - 1]->index)
+			continue;
+
+		if ((ptr - start) % len)
+			ptr += len - ((ptr - start) % len);
 		ptr += len;
 	}
 
 	len = chn->format.length / 8;
-	if (ptr % len)
-		ptr += len - (ptr % len);
+	if ((ptr - start) % len)
+		ptr += len - ((ptr - start) % len);
 	return (void *) ptr;
 }
 
