@@ -22,19 +22,15 @@
 /* Include public interface */
 #include "iio.h"
 
+#include "iio-config.h"
+
 #include <stdbool.h>
 
 #ifdef _MSC_BUILD
-#define snprintf sprintf_s
 #define inline __inline
-#endif
-#ifdef _WIN32
-#define strerror_r(err, buf, len) strerror_s(buf, len, err)
+#define iio_snprintf sprintf_s
 #else
-/* Visual Studio is unhappy with strdup(); so we make libiio use the C++
- * compliant _strdup() on Windows, and revert to the POSIX strdup() everywhere
- * else.*/
-#define _strdup strdup
+#define iio_snprintf snprintf
 #endif
 
 #ifdef _WIN32
@@ -49,6 +45,10 @@
 #   define __api
 #endif
 
+#ifdef WITH_MATLAB_BINDINGS_API
+#include "bindings/matlab/iio-wrapper.h"
+#endif
+
 #define ARRAY_SIZE(x) (sizeof(x) ? sizeof(x) / sizeof((x)[0]) : 0)
 #define BIT(x) (1 << (x))
 #define BIT_MASK(bit) BIT((bit) % 32)
@@ -89,6 +89,12 @@ static inline void *zalloc(size_t size)
 	return calloc(1, size);
 }
 
+enum iio_attr_type {
+	IIO_ATTR_TYPE_DEVICE = 0,
+	IIO_ATTR_TYPE_DEBUG,
+	IIO_ATTR_TYPE_BUFFER,
+};
+
 struct iio_backend_ops {
 	struct iio_context * (*clone)(const struct iio_context *ctx);
 	ssize_t (*read)(const struct iio_device *dev, void *dst, size_t len,
@@ -110,10 +116,10 @@ struct iio_backend_ops {
 			uint32_t *mask, size_t words);
 
 	ssize_t (*read_device_attr)(const struct iio_device *dev,
-			const char *attr, char *dst, size_t len, bool is_debug);
+			const char *attr, char *dst, size_t len, enum iio_attr_type);
 	ssize_t (*write_device_attr)(const struct iio_device *dev,
 			const char *attr, const char *src,
-			size_t len, bool is_debug);
+			size_t len, enum iio_attr_type);
 	ssize_t (*read_channel_attr)(const struct iio_channel *chn,
 			const char *attr, char *dst, size_t len);
 	ssize_t (*write_channel_attr)(const struct iio_channel *chn,
@@ -132,6 +138,11 @@ struct iio_backend_ops {
 	int (*set_timeout)(struct iio_context *ctx, unsigned int timeout);
 };
 
+/*
+ * If these structures are updated, the qsort functions defined in sort.c
+ * may need to be updated.
+ */
+
 struct iio_context_pdata;
 struct iio_device_pdata;
 struct iio_channel_pdata;
@@ -152,6 +163,10 @@ struct iio_context {
 	unsigned int nb_devices;
 
 	char *xml;
+
+	char **attrs;
+	char **values;
+	unsigned int nb_attrs;
 };
 
 struct iio_channel {
@@ -169,6 +184,8 @@ struct iio_channel {
 
 	struct iio_channel_attr *attrs;
 	unsigned int nb_attrs;
+
+	unsigned int number;
 };
 
 struct iio_device {
@@ -181,6 +198,9 @@ struct iio_device {
 	char **attrs;
 	unsigned int nb_attrs;
 
+	char **buffer_attrs;
+	unsigned int nb_buffer_attrs;
+
 	char **debug_attrs;
 	unsigned int nb_debug_attrs;
 
@@ -237,9 +257,6 @@ int iio_device_get_poll_fd(const struct iio_device *dev);
 
 int read_double(const char *str, double *val);
 int write_double(char *buf, size_t len, double val);
-#ifndef _WIN32
-int set_blocking_mode(int fd, bool blocking);
-#endif
 
 struct iio_context * local_create_context(void);
 struct iio_context * network_create_context(const char *hostname);
@@ -265,6 +282,11 @@ __api ssize_t iio_device_get_sample_size_mask(const struct iio_device *dev,
 void iio_channel_init_finalize(struct iio_channel *chn);
 unsigned int find_channel_modifier(const char *s, size_t *len_p);
 
+char *iio_strdup(const char *str);
+
+int iio_context_add_attr(struct iio_context *ctx,
+		const char *key, const char *value);
+
 #undef __api
 
 #endif /* __IIO_PRIVATE_H__ */
