@@ -1,19 +1,22 @@
 /*
- * libiio - Library for interfacing industrial I/O (IIO) devices
+ * iio_info - Part of Industrial I/O (IIO) utilities
  *
- * Copyright (C) 2014 Analog Devices, Inc.
+ * Copyright (C) 2014-2020 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  * */
 
 #include <errno.h>
@@ -25,6 +28,10 @@
 
 #define MY_NAME "iio_info"
 
+#ifdef _WIN32
+#define snprintf sprintf_s
+#endif
+
 enum backend {
 	LOCAL,
 	XML,
@@ -38,6 +45,7 @@ static const struct option options[] = {
 	  {"network", required_argument, 0, 'n'},
 	  {"uri", required_argument, 0, 'u'},
 	  {"scan", no_argument, 0, 's'},
+	  {"auto", no_argument, 0, 'a'},
 	  {0, 0, 0, 0},
 };
 
@@ -47,8 +55,29 @@ static const char *options_descriptions[] = {
 	"Use the network backend with the provided hostname.",
 	"Use the context at the provided URI.",
 	"Scan for available backends.",
+	"Scan for available contexts and if only one is available use it.",
 };
 
+/*
+ * internal buffers need to be big enough for attributes
+ * coming back from the kernel. Because of virtual memory,
+ * only the amount of ram that is needed is used.
+ */
+#define BUF_SIZE 16384
+
+static void * xmalloc(size_t n)
+{
+	void *p = malloc(n);
+
+	if (!p && n != 0) {
+		fprintf(stderr, MY_NAME
+				" fatal error: allocating %zu bytes failed\n",n);
+		exit(EXIT_FAILURE);
+	}
+
+	return p;
+}
+
 static void usage(void)
 {
 	unsigned int i;
@@ -100,18 +129,83 @@ err_free_ctx:
 	iio_scan_context_destroy(ctx);
 }
 
+static struct iio_context * autodetect_context(void)
+{
+	struct iio_scan_context *scan_ctx;
+	struct iio_context_info **info;
+	struct iio_context *ctx = NULL;
+	unsigned int i;
+	ssize_t ret;
+
+	scan_ctx = iio_create_scan_context(NULL, 0);
+	if (!scan_ctx) {
+		fprintf(stderr, "Unable to create scan context\n");
+		return NULL;
+	}
+
+	ret = iio_scan_context_get_info_list(scan_ctx, &info);
+	if (ret < 0) {
+		char err_str[1024];
+		iio_strerror(-ret, err_str, sizeof(err_str));
+		fprintf(stderr, "Scanning for IIO contexts failed: %s\n", err_str);
+		goto err_free_ctx;
+	}
+
+	if (ret == 0) {
+		printf("No IIO context found.\n");
+		goto err_free_info_list;
+	}
+
+	if (ret == 1) {
+		printf("Using auto-detected IIO context at URI \"%s\"\n",
+				iio_context_info_get_uri(info[0]));
+		ctx = iio_create_context_from_uri(iio_context_info_get_uri(info[0]));
+	} else {
+		fprintf(stderr, "Multiple contexts found. Please select one using --uri:\n");
+
+		for (i = 0; i < (size_t) ret; i++) {
+			fprintf(stderr, "\t%d: %s [%s]\n", i,
+				iio_context_info_get_description(info[i]),
+				iio_context_info_get_uri(info[i]));
+		}
+	}
+
+err_free_info_list:
+	iio_context_info_list_free(info);
+err_free_ctx:
+	iio_scan_context_destroy(scan_ctx);
+
+	return ctx;
+}
+
+static int dev_is_buffer_capable(const struct iio_device *dev)
+{
+	unsigned int i;
+
+	for (i = 0; i < iio_device_get_channels_count(dev); i++) {
+		struct iio_channel *chn = iio_device_get_channel(dev, i);
+
+		if (iio_channel_is_scan_element(chn))
+			return true;
+	}
+
+	return false;
+}
+
 int main(int argc, char **argv)
 {
 	struct iio_context *ctx;
-	int c, option_index = 0, arg_index = 0, xml_index = 0, ip_index = 0,
-	    uri_index = 0;
+	int c, option_index = 0;
+	const char *arg_uri = NULL;
+	const char *arg_ip = NULL;
+	const char *arg_xml = NULL;
 	enum backend backend = LOCAL;
-	bool do_scan = false;
-	unsigned int major, minor;
+	bool do_scan = false, detect_context = false;
+	unsigned int i, major, minor;
 	char git_tag[8];
 	int ret;
 
-	while ((c = getopt_long(argc, argv, "+hn:x:u:s",
+	while ((c = getopt_long(argc, argv, "+hn:x:u:sa",
 					options, &option_index)) != -1) {
 		switch (c) {
 		case 'h':
@@ -123,8 +217,7 @@ int main(int argc, char **argv)
 				return EXIT_FAILURE;
 			}
 			backend = NETWORK;
-			arg_index += 2;
-			ip_index = arg_index;
+			arg_ip = optarg;
 			break;
 		case 'x':
 			if (backend != LOCAL) {
@@ -132,11 +225,9 @@ int main(int argc, char **argv)
 				return EXIT_FAILURE;
 			}
 			backend = XML;
-			arg_index += 2;
-			xml_index = arg_index;
+			arg_xml = optarg;
 			break;
 		case 's':
-			arg_index += 1;
 			do_scan = true;
 			break;
 		case 'u':
@@ -145,15 +236,17 @@ int main(int argc, char **argv)
 				return EXIT_FAILURE;
 			}
 			backend = AUTO;
-			arg_index += 2;
-			uri_index = arg_index;
+			arg_uri = optarg;
+			break;
+		case 'a':
+			detect_context = true;
 			break;
 		case '?':
 			return EXIT_FAILURE;
 		}
 	}
 
-	if (arg_index >= argc) {
+	if (optind != argc) {
 		fprintf(stderr, "Incorrect number of arguments.\n\n");
 		usage();
 		return EXIT_FAILURE;
@@ -162,22 +255,36 @@ int main(int argc, char **argv)
 	iio_library_get_version(&major, &minor, git_tag);
 	printf("Library version: %u.%u (git tag: %s)\n", major, minor, git_tag);
 
+	printf("Compiled with backends:");
+	for (i = 0; i < iio_get_backends_count(); i++)
+		printf(" %s", iio_get_backend(i));
+	printf("\n");
+
 	if (do_scan) {
 		scan();
 		return EXIT_SUCCESS;
 	}
 
-	if (backend == XML)
-		ctx = iio_create_xml_context(argv[xml_index]);
+	if (detect_context)
+		ctx = autodetect_context();
+	else if (backend == XML)
+		ctx = iio_create_xml_context(arg_xml);
 	else if (backend == NETWORK)
-		ctx = iio_create_network_context(argv[ip_index]);
+		ctx = iio_create_network_context(arg_ip);
 	else if (backend == AUTO)
-		ctx = iio_create_context_from_uri(argv[uri_index]);
+		ctx = iio_create_context_from_uri(arg_uri);
 	else
 		ctx = iio_create_default_context();
 
 	if (!ctx) {
-		fprintf(stderr, "Unable to create IIO context\n");
+		if (!detect_context) {
+			char buf[1024];
+
+			iio_strerror(errno, buf, sizeof(buf));
+			fprintf(stderr, "Unable to create IIO context: %s\n",
+					buf);
+		}
+
 		return EXIT_FAILURE;
 	}
 
@@ -194,14 +301,30 @@ int main(int argc, char **argv)
 	printf("Backend description string: %s\n",
 			iio_context_get_description(ctx));
 
+	unsigned int nb_ctx_attrs = iio_context_get_attrs_count(ctx);
+	if (nb_ctx_attrs > 0)
+		printf("IIO context has %u attributes:\n", nb_ctx_attrs);
+
+	for (i = 0; i < nb_ctx_attrs; i++) {
+		const char *key, *value;
+
+		iio_context_get_attr(ctx, i, &key, &value);
+		printf("\t%s: %s\n", key, value);
+	}
+
 	unsigned int nb_devices = iio_context_get_devices_count(ctx);
 	printf("IIO context has %u devices:\n", nb_devices);
+	char *buf = xmalloc(BUF_SIZE);
 
-	unsigned int i;
 	for (i = 0; i < nb_devices; i++) {
 		const struct iio_device *dev = iio_context_get_device(ctx, i);
 		const char *name = iio_device_get_name(dev);
-		printf("\t%s: %s\n", iio_device_get_id(dev), name ? name : "" );
+		printf("\t%s:", iio_device_get_id(dev));
+		if (name)
+			printf(" %s", name);
+		if (dev_is_buffer_capable(dev))
+			printf(" (buffer capable)");
+		printf("\n");
 
 		unsigned int nb_channels = iio_device_get_channels_count(dev);
 		printf("\t\t%u channels found:\n", nb_channels);
@@ -217,10 +340,36 @@ int main(int argc, char **argv)
 				type_name = "input";
 
 			name = iio_channel_get_name(ch);
-			printf("\t\t\t%s: %s (%s)\n",
+			printf("\t\t\t%s: %s (%s",
 					iio_channel_get_id(ch),
 					name ? name : "", type_name);
 
+			if (iio_channel_get_type(ch) == IIO_CHAN_TYPE_UNKNOWN)
+				printf(", WARN:iio_channel_get_type()=UNKNOWN");
+
+			if (iio_channel_is_scan_element(ch)) {
+				const struct iio_data_format *format =
+					iio_channel_get_data_format(ch);
+				char sign = format->is_signed ? 's' : 'u';
+				char repeat[12] = "";
+
+				if (format->is_fully_defined)
+					sign += 'A' - 'a';
+
+				if (format->repeat > 1)
+					snprintf(repeat, sizeof(repeat), "X%u",
+						format->repeat);
+
+				printf(", index: %lu, format: %ce:%c%u/%u%s>>%u)\n",
+					iio_channel_get_index(ch),
+					format->is_be ? 'b' : 'l',
+					sign, format->bits,
+					format->length, repeat,
+					format->shift);
+			} else {
+				printf(")\n");
+			}
+
 			unsigned int nb_attrs = iio_channel_get_attrs_count(ch);
 			if (!nb_attrs)
 				continue;
@@ -231,45 +380,59 @@ int main(int argc, char **argv)
 			unsigned int k;
 			for (k = 0; k < nb_attrs; k++) {
 				const char *attr = iio_channel_get_attr(ch, k);
-				char buf[1024];
 				ret = (int) iio_channel_attr_read(ch,
-						attr, buf, sizeof(buf));
-				if (ret > 0) {
-					printf("\t\t\t\tattr %u: %s"
-							" value: %s\n", k,
-							attr, buf);
-				} else if (ret == -ENOSYS) {
-					printf("\t\t\t\tattr %u: %s\n",
-							k, attr);
-				} else {
-					iio_strerror(-ret, buf, sizeof(buf));
+						attr, buf, BUF_SIZE);
 
-					fprintf(stderr, "Unable to read attribute %s: %s\n",
-							attr, buf);
+				printf("\t\t\t\tattr %2u: %s ", k, attr);
+
+				if (ret > 0) {
+					printf("value: %s\n", buf);
+				} else {
+					iio_strerror(-ret, buf, BUF_SIZE);
+					printf("ERROR: %s (%i)\n", buf, ret);
 				}
 			}
 		}
 
 		unsigned int nb_attrs = iio_device_get_attrs_count(dev);
-		if (!nb_attrs)
-			continue;
+		if (nb_attrs) {
+			printf("\t\t%u device-specific attributes found:\n",
+					nb_attrs);
+			for (j = 0; j < nb_attrs; j++) {
+				const char *attr = iio_device_get_attr(dev, j);
+				ret = (int) iio_device_attr_read(dev,
+						attr, buf, BUF_SIZE);
 
-		printf("\t\t%u device-specific attributes found:\n", nb_attrs);
-		for (j = 0; j < nb_attrs; j++) {
-			const char *attr = iio_device_get_attr(dev, j);
-			char buf[1024];
-			ret = (int) iio_device_attr_read(dev,
-					attr, buf, sizeof(buf));
-			if (ret > 0) {
-				printf("\t\t\t\tattr %u: %s value: %s\n", j,
-						attr, buf);
-			} else if (ret == -ENOSYS) {
-				printf("\t\t\t\tattr %u: %s\n", j, attr);
-			} else {
-				iio_strerror(-ret, buf, sizeof(buf));
+				printf("\t\t\t\tattr %2u: %s ",
+						j, attr);
 
-				fprintf(stderr, "Unable to read attribute %s: %s\n",
-						attr, buf);
+				if (ret > 0) {
+					printf("value: %s\n", buf);
+				} else {
+					iio_strerror(-ret, buf, BUF_SIZE);
+					printf("ERROR: %s (%i)\n", buf, ret);
+				}
+			}
+		}
+
+		nb_attrs = iio_device_get_buffer_attrs_count(dev);
+		if (nb_attrs) {
+			printf("\t\t%u buffer-specific attributes found:\n",
+					nb_attrs);
+			for (j = 0; j < nb_attrs; j++) {
+				const char *attr = iio_device_get_buffer_attr(dev, j);
+				ret = (int) iio_device_buffer_attr_read(dev,
+						attr, buf, BUF_SIZE);
+
+				printf("\t\t\t\tattr %2u: %s ",
+						j, attr);
+
+				if (ret > 0) {
+					printf("value: %s\n", buf);
+				} else {
+					iio_strerror(-ret, buf, BUF_SIZE);
+					printf("ERROR: %s (%i)\n", buf, ret);
+				}
 			}
 		}
 
@@ -279,16 +442,17 @@ int main(int argc, char **argv)
 			for (j = 0; j < nb_attrs; j++) {
 				const char *attr =
 					iio_device_get_debug_attr(dev, j);
-				char buf[1024];
 
 				ret = (int) iio_device_debug_attr_read(dev,
-						attr, buf, sizeof(buf));
-				if (ret > 0)
-					printf("\t\t\t\tdebug attr %u: %s value: %s\n",
-							j, attr, buf);
-				else if (ret == -ENOSYS)
-					printf("\t\t\t\tdebug attr %u: %s\n", j,
-							attr);
+						attr, buf, BUF_SIZE);
+				printf("\t\t\t\tdebug attr %2u: %s ",
+						j, attr);
+				if (ret > 0) {
+					printf("value: %s\n", buf);
+				} else {
+					iio_strerror(-ret, buf, BUF_SIZE);
+					printf("ERROR: %s (%i)\n", buf, ret);
+				}
 			}
 		}
 
@@ -306,6 +470,7 @@ int main(int argc, char **argv)
 		}
 	}
 
+	free(buf);
 	iio_context_destroy(ctx);
 	return EXIT_SUCCESS;
 }
